
!=======================================================================
! CellML file:   luo_rudy_I_model_1991.cellml
! CellML model:  luo_rudy_I_model_1991
! Date and time: 19/01/2009 at 11:45:25
!-----------------------------------------------------------------------
! Conversion from CellML 1.0 to Fortran 77 was done using COR (0.9.31.1165)
!    Copyright 2002-2009 Dr Alan Garny
!    http://COR.physiol.ox.ac.uk/ - COR@physiol.ox.ac.uk
!-----------------------------------------------------------------------
! http://www.CellML.org/
!=======================================================================

! Modified by F. Maggio (October 2009) to comply with the semi-implicit
! time marching scheme:
!
! i_ion = i_ion(v,w,c)
! dw/dt = R(v,w)
! dc/dt = S(v,w,c)
! and then:
! w^(n+1) = w^n + deltat*R(v^n,w^(n+1))
! c^(n+1) = c^n + deltat*S(v^n,w^(n+1),c^n)
!
! described in:
!         Colli Franzone P, Pavarino L, Savare' G, "Computational 
!         electrocardiology: mathematical and numerical modeling" in 
!         Quarteroni A, Formaggia L, Veneziani A (eds.), "Complex systems in 
!         biomedicine", Springer-Verlag, Milano 2006:

      !-----------------------------------------------------------------
      ! Initialisation
      !-----------------------------------------------------------------

      SUBROUTINE INIT
         IMPLICIT NONE

         real(8) :: Y(8)
         COMMON /Y/ Y

         CHARACTER YNames(8)*3
         COMMON /YNames/ YNames
         CHARACTER YUnits(8)*13
         COMMON /YUnits/ YUnits
         CHARACTER YComponents(8)*39
         COMMON /YComponents/ YComponents

         real(8) :: E_b   ! millivolt (in background_current)
         real(8) :: g_b   ! milliS_per_cm2 (in background_current)
         real(8) :: g_Na   ! milliS_per_cm2 (in fast_sodium_current)
         real(8) :: Ki   ! millimolar (in ionic_concentrations)
         real(8) :: Ko   ! millimolar (in ionic_concentrations)
         real(8) :: Nai   ! millimolar (in ionic_concentrations)
         real(8) :: Nao   ! millimolar (in ionic_concentrations)
         real(8) :: C   ! microF_per_cm2 (in membrane)
         real(8) :: F   ! coulomb_per_mole (in membrane)
         real(8) :: R   ! joule_per_kilomole_kelvin (in membrane)
         real(8) :: T   ! kelvin (in membrane)
         real(8) :: g_Kp   ! milliS_per_cm2 (in plateau_potassium_current)
         real(8) :: PR_NaK   ! dimensionless (in time_dependent_potassium_current)

         COMMON /CONSTANTS001/ E_b, g_b, g_Na, Ki, Ko, Nai, Nao, C, F, R,      &
                               T, g_Kp, PR_NaK

         real(8) :: i_b   ! microA_per_cm2 (in background_current)
         real(8) :: E_Na   ! millivolt (in fast_sodium_current)
         real(8) :: alpha_h   ! per_millisecond (in fast_sodium_current_h_gate)
         real(8) :: beta_h   ! per_millisecond (in fast_sodium_current_h_gate)
         real(8) :: alpha_j   ! per_millisecond (in fast_sodium_current_j_gate)
         real(8) :: beta_j   ! per_millisecond (in fast_sodium_current_j_gate)
         real(8) :: alpha_m   ! per_millisecond (in fast_sodium_current_m_gate)
         real(8) :: beta_m   ! per_millisecond (in fast_sodium_current_m_gate)
         real(8) :: i_Na   ! microA_per_cm2 (in fast_sodium_current)
         real(8) :: E_Kp   ! millivolt (in plateau_potassium_current)
         real(8) :: Kp   ! dimensionless (in plateau_potassium_current)
         real(8) :: i_Kp   ! microA_per_cm2 (in plateau_potassium_current)
         real(8) :: E_si   ! millivolt (in slow_inward_current)
         real(8) :: alpha_d   ! per_millisecond (in slow_inward_current_d_gate)
         real(8) :: beta_d   ! per_millisecond (in slow_inward_current_d_gate)
         real(8) :: alpha_f   ! per_millisecond (in slow_inward_current_f_gate)
         real(8) :: beta_f   ! per_millisecond (in slow_inward_current_f_gate)
         real(8) :: i_si   ! microA_per_cm2 (in slow_inward_current)
         real(8) :: E_K   ! millivolt (in time_dependent_potassium_current)
         real(8) :: alpha_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
         real(8) :: beta_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
         real(8) :: Xi   ! dimensionless (in time_dependent_potassium_current_Xi_gate)
         real(8) :: g_K   ! milliS_per_cm2 (in time_dependent_potassium_current)
         real(8) :: i_K   ! microA_per_cm2 (in time_dependent_potassium_current)
         real(8) :: E_K1   ! millivolt (in time_independent_potassium_current)
         real(8) :: K1_infinity   ! dimensionless (in time_independent_potassium_current_K1_gate)
         real(8) :: alpha_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
         real(8) :: beta_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
         real(8) :: g_K1   ! milliS_per_cm2 (in time_independent_potassium_current)
         real(8) :: i_K1   ! microA_per_cm2 (in time_independent_potassium_current)

         COMMON /COMPVARS001/ i_b, E_Na, alpha_h, beta_h, alpha_j, beta_j,    &
		   alpha_m, beta_m, i_Na, E_Kp, Kp, i_Kp, E_si, alpha_d,      &
		   beta_d, alpha_f, beta_f, i_si, E_K, alpha_X, beta_X, Xi, g_K, i_K, &
           E_K1, K1_infinity, alpha_K1, beta_K1, g_K1, i_K1
         !--------------------------------------------------------------
         ! State variables
         !--------------------------------------------------------------

         Y(1) = 0.9804713D0   ! h (dimensionless) (in fast_sodium_current_h_gate)
         Y(2) = 0.98767124D0   ! j (dimensionless) (in fast_sodium_current_j_gate)
         Y(3) = 0.00187018D0   ! m (dimensionless) (in fast_sodium_current_m_gate)
         Y(4) = 0.0002D0   ! Cai (millimolar) (in intracellular_calcium_concentration)
         Y(5) = -83.853D0   ! V (millivolt) (in membrane)
         Y(6) = 0.00316354D0   ! d (dimensionless) (in slow_inward_current_d_gate)
         Y(7) = 0.99427859D0   ! f (dimensionless) (in slow_inward_current_f_gate)
         Y(8) = 0.16647703D0   ! X (dimensionless) (in time_dependent_potassium_current_X_gate)

         YNames(1) = 'h'
         YNames(2) = 'j'
         YNames(3) = 'm'
         YNames(4) = 'Cai'
         YNames(5) = 'V'
         YNames(6) = 'd'
         YNames(7) = 'f'
         YNames(8) = 'X'

         YUnits(1) = 'dimensionless'
         YUnits(2) = 'dimensionless'
         YUnits(3) = 'dimensionless'
         YUnits(4) = 'millimolar'
         YUnits(5) = 'millivolt'
         YUnits(6) = 'dimensionless'
         YUnits(7) = 'dimensionless'
         YUnits(8) = 'dimensionless'

         YComponents(1) = 'fast_sodium_current_h_gate'
         YComponents(2) = 'fast_sodium_current_j_gate'
         YComponents(3) = 'fast_sodium_current_m_gate'
         YComponents(4) = 'intracellular_calcium_concentration'
         YComponents(5) = 'membrane'
         YComponents(6) = 'slow_inward_current_d_gate'
         YComponents(7) = 'slow_inward_current_f_gate'
         YComponents(8) = 'time_dependent_potassium_current_X_gate'

         !--------------------------------------------------------------
         ! Constants
         !--------------------------------------------------------------

         E_b = -59.87D0   ! millivolt (in background_current)
         g_b = 0.03921D0   ! milliS_per_cm2 (in background_current)
         g_Na = 23.0D0   ! milliS_per_cm2 (in fast_sodium_current)
         Ki = 145.0D0   ! millimolar (in ionic_concentrations)
         Ko = 5.4D0   ! millimolar (in ionic_concentrations)
         Nai = 18.0D0   ! millimolar (in ionic_concentrations)
         Nao = 140.0D0   ! millimolar (in ionic_concentrations)
         C = 1.0D0   ! microF_per_cm2 (in membrane)
         F = 96484.6D0   ! coulomb_per_mole (in membrane)
         R = 8314.0D0   ! joule_per_kilomole_kelvin (in membrane)
         T = 310.0D0   ! kelvin (in membrane)
         g_Kp = 0.0183D0   ! milliS_per_cm2 (in plateau_potassium_current)
         PR_NaK = 0.01833D0   ! dimensionless (in time_dependent_potassium_current)

         !--------------------------------------------------------------
         ! Computed variables
         !--------------------------------------------------------------

         E_Na = R*T/F*DLOG(Nao/Nai)
         g_K = 0.282D0*DSQRT(Ko/5.4D0)
         E_K = R*T/F*DLOG((Ko+PR_NaK*Nao)/(Ki+PR_NaK*Nai))
         g_K1 = 0.6047D0*DSQRT(Ko/5.4D0)
         E_K1 = R*T/F*DLOG(Ko/Ki)
         E_Kp = E_K1

         RETURN
      END

      !-----------------------------------------------------------------
      ! Computation
      !-----------------------------------------------------------------

      SUBROUTINE COMPUTE_V(time,deltat)
         IMPLICIT NONE

         real(8) :: time, deltat            ! millis
         real(8) :: v_dot

         real(8) :: Y(8), y_new(8)
         COMMON /Y/ Y
         real(8) :: DY(8)

         real(8) :: E_b   ! millivolt (in background_current)
         real(8) :: g_b   ! milliS_per_cm2 (in background_current)
         real(8) :: g_Na   ! milliS_per_cm2 (in fast_sodium_current)
         real(8) :: Ki   ! millimolar (in ionic_concentrations)
         real(8) :: Ko   ! millimolar (in ionic_concentrations)
         real(8) :: Nai   ! millimolar (in ionic_concentrations)
         real(8) :: Nao   ! millimolar (in ionic_concentrations)
         real(8) :: C   ! microF_per_cm2 (in membrane)
         real(8) :: F   ! coulomb_per_mole (in membrane)
         real(8) :: R   ! joule_per_kilomole_kelvin (in membrane)
         real(8) :: T   ! kelvin (in membrane)
         real(8) :: g_Kp   ! milliS_per_cm2 (in plateau_potassium_current)
         real(8) :: PR_NaK   ! dimensionless (in time_dependent_potassium_current)

         COMMON /CONSTANTS001/ E_b, g_b, g_Na, Ki, Ko, Nai, Nao, C, F, R,      &
		                       T, g_Kp, PR_NaK

         real(8) :: i_b   ! microA_per_cm2 (in background_current)
         real(8) :: E_Na   ! millivolt (in fast_sodium_current)
         real(8) :: alpha_h   ! per_millisecond (in fast_sodium_current_h_gate)
         real(8) :: beta_h   ! per_millisecond (in fast_sodium_current_h_gate)
         real(8) :: alpha_j   ! per_millisecond (in fast_sodium_current_j_gate)
         real(8) :: beta_j   ! per_millisecond (in fast_sodium_current_j_gate)
         real(8) :: alpha_m   ! per_millisecond (in fast_sodium_current_m_gate)
         real(8) :: beta_m   ! per_millisecond (in fast_sodium_current_m_gate)
         real(8) :: i_Na   ! microA_per_cm2 (in fast_sodium_current)
         real(8) :: E_Kp   ! millivolt (in plateau_potassium_current)
         real(8) :: Kp   ! dimensionless (in plateau_potassium_current)
         real(8) :: i_Kp   ! microA_per_cm2 (in plateau_potassium_current)
         real(8) :: E_si   ! millivolt (in slow_inward_current)
         real(8) :: alpha_d   ! per_millisecond (in slow_inward_current_d_gate)
         real(8) :: beta_d   ! per_millisecond (in slow_inward_current_d_gate)
         real(8) :: alpha_f   ! per_millisecond (in slow_inward_current_f_gate)
         real(8) :: beta_f   ! per_millisecond (in slow_inward_current_f_gate)
         real(8) :: i_si   ! microA_per_cm2 (in slow_inward_current)
         real(8) :: E_K   ! millivolt (in time_dependent_potassium_current)
         real(8) :: alpha_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
         real(8) :: beta_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
         real(8) :: Xi   ! dimensionless (in time_dependent_potassium_current_Xi_gate)
         real(8) :: g_K   ! milliS_per_cm2 (in time_dependent_potassium_current)
         real(8) :: i_K   ! microA_per_cm2 (in time_dependent_potassium_current)
         real(8) :: E_K1   ! millivolt (in time_independent_potassium_current)
         real(8) :: K1_infinity   ! dimensionless (in time_independent_potassium_current_K1_gate)
         real(8) :: alpha_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
         real(8) :: beta_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
         real(8) :: g_K1   ! milliS_per_cm2 (in time_independent_potassium_current)
         real(8) :: i_K1   ! microA_per_cm2 (in time_independent_potassium_current)
         real(8) :: I_stim, stim_amplitude = -25.5D0   ! microA_per_cm2 (in membrane)
         real(8), external :: hhh

         COMMON /COMPVARS001/ i_b, E_Na, alpha_h, beta_h, alpha_j, beta_j,    &
		   alpha_m, beta_m, i_Na, E_Kp, Kp, i_Kp, E_si, alpha_d,      &
		   beta_d, alpha_f, beta_f, i_si, E_K, alpha_X, beta_X, Xi, g_K,      &
		   i_K,E_K1, K1_infinity, alpha_K1, beta_K1, g_K1, i_K1

!        w_1 = h:
         IF (Y(5) .LT. -40.0D0) THEN
            alpha_h = 0.135D0*DEXP((80.0D0+Y(5))/(-6.8D0))
         ELSE
            alpha_h = 0.0D0
         END IF
         IF (Y(5) .LT. -40.0D0) THEN
            beta_h = 3.56D0*DEXP(0.079D0*Y(5))+310000.0D0*DEXP(0.35D0*Y(5))
         ELSE
            beta_h = 1.0D0/(0.13D0*(1.0D0+DEXP((Y(5)+10.66D0)/(-11.1D0))))
         END IF
         y(1)=(y(1)/deltat+alpha_h)/(1.d0/deltat+alpha_h+beta_h)

!        w_2 = j:
         IF (Y(5) .LT. -40.0D0) THEN
            alpha_j = (-127140.0D0*DEXP(0.2444D0*Y(5))-0.00003474D0*    &
			DEXP(-0.04391D0*Y(5)))*(Y(5)+37.78D0)/(1.0D0+DEXP(0.311D0*  &
			(Y(5)+79.23D0)))
         ELSE
            alpha_j = 0.0D0
         END IF
         IF (Y(5) .LT. -40.0D0) THEN
            beta_j = 0.1212D0*DEXP(-0.01052D0*Y(5))/(1.0D0+DEXP(-0.1378D0*   &
			         (Y(5)+40.14D0)))
         ELSE
            beta_j = 0.3D0*DEXP(-0.0000002535D0*Y(5))/(1.0D0+DEXP(-0.1D0*    &
			         (Y(5)+32.0D0)))
         END IF      
	     y(2)=(y(2)/deltat+alpha_j)/(1.d0/deltat+alpha_j+beta_j)

!        w_3 = m:         
		 alpha_m = 0.32D0*(Y(5)+47.13D0)/(1.0D0-DEXP(-0.1D0*(Y(5)+47.13D0)))
         beta_m = 0.08D0*DEXP(-Y(5)/11.0D0)
         y(3)=(y(3)/deltat+alpha_m)/(1.d0/deltat+alpha_m+beta_m)

!        w_6 = d:
         alpha_d = 0.095D0*DEXP(-0.01D0*(Y(5)-5.0D0))/(1.0D0+   &
		           DEXP(-0.072D0*(Y(5)-5.0D0)))
         beta_d = 0.07D0*DEXP(-0.017D0*(Y(5)+44.0D0))/(1.0D0+   &
		          DEXP(0.05D0*(Y(5)+44.0D0)))
         y(6)=(y(6)/deltat+alpha_d)/(1.d0/deltat+alpha_d+beta_d)

!        w_7 = f:
         alpha_f = 0.012D0*DEXP(-0.008D0*(Y(5)+28.0D0))/(1.0D0+ &
		           DEXP(0.15D0*(Y(5)+28.0D0)))
         beta_f = 0.0065D0*DEXP(-0.02D0*(Y(5)+30.0D0))/(1.0D0+  &
		          DEXP(-0.2D0*(Y(5)+30.0D0)))
         y(7)=(y(7)/deltat+alpha_f)/(1.d0/deltat+alpha_f+beta_f)

!        w_8 = X:
         alpha_X = 0.0005D0*DEXP(0.083D0*(Y(5)+50.0D0))/(1.0D0+ &
		           DEXP(0.057D0*(Y(5)+50.0D0)))
         beta_X = 0.0013D0*DEXP(-0.06D0*(Y(5)+20.0D0))/(1.0D0+  &
		          DEXP(-0.04D0*(Y(5)+20.0D0)))
         y(8)=(y(8)/deltat+alpha_X)/(1.d0/deltat+alpha_X+beta_X)

!        c_1 = Cai:
         E_si = 7.7D0-13.0287D0*DLOG(Y(4)/1.0D0)
         i_si = 0.09D0*Y(6)*Y(7)*(Y(5)-E_si)
         DY(4) = -0.0001D0/1.0D0*i_si+0.07D0*(0.0001D0-Y(4))
         y(4)=y(4)+deltat*dy(4)

         IF (Y(5) .GT. -100.0D0) THEN
            Xi = 2.837D0*(DEXP(0.04D0*(Y(5)+77.0D0))-1.0D0)/((Y(5)+77.0D0)*   &
			     DEXP(0.04D0*(Y(5)+35.0D0)))
         ELSE
            Xi = 1.0D0
         END IF

         i_b = g_b*(Y(5)-E_b)
         i_Na = g_Na*Y(3)**3.0D0*Y(1)*Y(2)*(Y(5)-E_Na)
         i_K = g_K*Y(8)*Xi*(Y(5)-E_K)
         alpha_K1 = 1.02D0/(1.0D0+DEXP(0.2385D0*(Y(5)-E_K1-59.215D0)))
         beta_K1 = (0.49124D0*DEXP(0.08032D0*(Y(5)+5.476D0-E_K1))+1.0D0*    &
		 DEXP(0.06175D0*(Y(5)-(E_K1+594.31D0))))/(1.0D0+DEXP(-0.5143D0*     &
		 (Y(5)-E_K1+4.753D0)))
         K1_infinity = alpha_K1/(alpha_K1+beta_K1)
         i_K1 = g_K1*K1_infinity*(Y(5)-E_K1)
         Kp = 1.0D0/(1.0D0+DEXP((7.488D0-Y(5))/5.98D0))
         i_Kp = g_Kp*Kp*(Y(5)-E_Kp)
		 I_stim=stim_amplitude*hhh(time)
         v_dot = -(i_Na+i_si+i_K+i_K1+i_Kp+i_b+I_stim)/C
write(137,*) time, v_dot
         y(5)=y(5)+deltat*v_dot

         RETURN
      END SUBROUTINE COMPUTE_V


!     --------------------------------------------------------------------------


      SUBROUTINE COMPUTE_I(time,deltat)
         IMPLICIT NONE

         real(8) :: time, deltat
         ! time: time (millisecond)

         real(8) :: Y(8), y_new(8)
         COMMON /Y/ Y
         real(8) :: DY(8)

         real(8) :: E_b   ! millivolt (in background_current)
         real(8) :: g_b   ! milliS_per_cm2 (in background_current)
         real(8) :: g_Na   ! milliS_per_cm2 (in fast_sodium_current)
         real(8) :: Ki   ! millimolar (in ionic_concentrations)
         real(8) :: Ko   ! millimolar (in ionic_concentrations)
         real(8) :: Nai   ! millimolar (in ionic_concentrations)
         real(8) :: Nao   ! millimolar (in ionic_concentrations)
         real(8) :: C   ! microF_per_cm2 (in membrane)
         real(8) :: F   ! coulomb_per_mole (in membrane)
         real(8) :: R   ! joule_per_kilomole_kelvin (in membrane)
         real(8) :: T   ! kelvin (in membrane)
         real(8) :: g_Kp   ! milliS_per_cm2 (in plateau_potassium_current)
         real(8) :: PR_NaK   ! dimensionless (in time_dependent_potassium_current)

         COMMON /CONSTANTS001/ E_b, g_b, g_Na, Ki, Ko, Nai, Nao, C, F, R,      &
		                       T, g_Kp, PR_NaK

         real(8) :: i_b   ! microA_per_cm2 (in background_current)
         real(8) :: E_Na   ! millivolt (in fast_sodium_current)
         real(8) :: alpha_h   ! per_millisecond (in fast_sodium_current_h_gate)
         real(8) :: beta_h   ! per_millisecond (in fast_sodium_current_h_gate)
         real(8) :: alpha_j   ! per_millisecond (in fast_sodium_current_j_gate)
         real(8) :: beta_j   ! per_millisecond (in fast_sodium_current_j_gate)
         real(8) :: alpha_m   ! per_millisecond (in fast_sodium_current_m_gate)
         real(8) :: beta_m   ! per_millisecond (in fast_sodium_current_m_gate)
         real(8) :: i_Na   ! microA_per_cm2 (in fast_sodium_current)
         real(8) :: E_Kp   ! millivolt (in plateau_potassium_current)
         real(8) :: Kp   ! dimensionless (in plateau_potassium_current)
         real(8) :: i_Kp   ! microA_per_cm2 (in plateau_potassium_current)
         real(8) :: E_si   ! millivolt (in slow_inward_current)
         real(8) :: alpha_d   ! per_millisecond (in slow_inward_current_d_gate)
         real(8) :: beta_d   ! per_millisecond (in slow_inward_current_d_gate)
         real(8) :: alpha_f   ! per_millisecond (in slow_inward_current_f_gate)
         real(8) :: beta_f   ! per_millisecond (in slow_inward_current_f_gate)
         real(8) :: i_si   ! microA_per_cm2 (in slow_inward_current)
         real(8) :: E_K   ! millivolt (in time_dependent_potassium_current)
         real(8) :: alpha_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
         real(8) :: beta_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
         real(8) :: Xi   ! dimensionless (in time_dependent_potassium_current_Xi_gate)
         real(8) :: g_K   ! milliS_per_cm2 (in time_dependent_potassium_current)
         real(8) :: i_K   ! microA_per_cm2 (in time_dependent_potassium_current)
         real(8) :: E_K1   ! millivolt (in time_independent_potassium_current)
         real(8) :: K1_infinity   ! dimensionless (in time_independent_potassium_current_K1_gate)
         real(8) :: alpha_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
         real(8) :: beta_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
         real(8) :: g_K1   ! milliS_per_cm2 (in time_independent_potassium_current)
         real(8) :: i_K1   ! microA_per_cm2 (in time_independent_potassium_current)

         COMMON /COMPVARS001/ i_b, E_Na, alpha_h, beta_h, alpha_j, beta_j,    &
		   alpha_m, beta_m, i_Na, E_Kp, Kp, i_Kp, E_si, alpha_d,      &
		   beta_d, alpha_f, beta_f, i_si, E_K, alpha_X, beta_X, Xi, g_K,      &
		   i_K,E_K1, K1_infinity, alpha_K1, beta_K1, g_K1, i_K1

!        w_1 = h:
         IF (Y(5) .LT. -40.0D0) THEN
            alpha_h = 0.135D0*DEXP((80.0D0+Y(5))/(-6.8D0))
         ELSE
            alpha_h = 0.0D0
         END IF
         IF (Y(5) .LT. -40.0D0) THEN
            beta_h = 3.56D0*DEXP(0.079D0*Y(5))+310000.0D0*DEXP(0.35D0*Y(5))
         ELSE
            beta_h = 1.0D0/(0.13D0*(1.0D0+DEXP((Y(5)+10.66D0)/(-11.1D0))))
         END IF
         y(1)=(y(1)/deltat+alpha_h)/(1.d0/deltat+alpha_h+beta_h)

!        w_2 = j:
         IF (Y(5) .LT. -40.0D0) THEN
            alpha_j = (-127140.0D0*DEXP(0.2444D0*Y(5))-0.00003474D0*    &
			DEXP(-0.04391D0*Y(5)))*(Y(5)+37.78D0)/(1.0D0+DEXP(0.311D0*  &
			(Y(5)+79.23D0)))
         ELSE
            alpha_j = 0.0D0
         END IF
         IF (Y(5) .LT. -40.0D0) THEN
            beta_j = 0.1212D0*DEXP(-0.01052D0*Y(5))/(1.0D0+DEXP(-0.1378D0*   &
			         (Y(5)+40.14D0)))
         ELSE
            beta_j = 0.3D0*DEXP(-0.0000002535D0*Y(5))/(1.0D0+DEXP(-0.1D0*    &
			         (Y(5)+32.0D0)))
         END IF      
	     y(2)=(y(2)/deltat+alpha_j)/(1.d0/deltat+alpha_j+beta_j)

!        w_3 = m:         
		 alpha_m = 0.32D0*(Y(5)+47.13D0)/(1.0D0-DEXP(-0.1D0*(Y(5)+47.13D0)))
         beta_m = 0.08D0*DEXP(-Y(5)/11.0D0)
         y(3)=(y(3)/deltat+alpha_m)/(1.d0/deltat+alpha_m+beta_m)

!        w_6 = d:
         alpha_d = 0.095D0*DEXP(-0.01D0*(Y(5)-5.0D0))/(1.0D0+   &
		           DEXP(-0.072D0*(Y(5)-5.0D0)))
         beta_d = 0.07D0*DEXP(-0.017D0*(Y(5)+44.0D0))/(1.0D0+   &
		          DEXP(0.05D0*(Y(5)+44.0D0)))
         y(6)=(y(6)/deltat+alpha_d)/(1.d0/deltat+alpha_d+beta_d)

!        w_7 = f:
         alpha_f = 0.012D0*DEXP(-0.008D0*(Y(5)+28.0D0))/(1.0D0+ &
		           DEXP(0.15D0*(Y(5)+28.0D0)))
         beta_f = 0.0065D0*DEXP(-0.02D0*(Y(5)+30.0D0))/(1.0D0+  &
		          DEXP(-0.2D0*(Y(5)+30.0D0)))
         y(7)=(y(7)/deltat+alpha_f)/(1.d0/deltat+alpha_f+beta_f)

!        w_8 = X:
         alpha_X = 0.0005D0*DEXP(0.083D0*(Y(5)+50.0D0))/(1.0D0+ &
		           DEXP(0.057D0*(Y(5)+50.0D0)))
         beta_X = 0.0013D0*DEXP(-0.06D0*(Y(5)+20.0D0))/(1.0D0+  &
		          DEXP(-0.04D0*(Y(5)+20.0D0)))
         y(8)=(y(8)/deltat+alpha_X)/(1.d0/deltat+alpha_X+beta_X)

!        c_1 = Cai:
         E_si = 7.7D0-13.0287D0*DLOG(Y(4)/1.0D0)
         i_si = 0.09D0*Y(6)*Y(7)*(Y(5)-E_si)
         DY(4) = -0.0001D0/1.0D0*i_si+0.07D0*(0.0001D0-Y(4))
         y(4)=y(4)+deltat*dy(4)

         IF (Y(5) .GT. -100.0D0) THEN
            Xi = 2.837D0*(DEXP(0.04D0*(Y(5)+77.0D0))-1.0D0)/((Y(5)+77.0D0)*   &
			     DEXP(0.04D0*(Y(5)+35.0D0)))
         ELSE
            Xi = 1.0D0
         END IF

         i_b = g_b*(Y(5)-E_b)
         i_Na = g_Na*Y(3)**3.0D0*Y(1)*Y(2)*(Y(5)-E_Na)
         i_K = g_K*Y(8)*Xi*(Y(5)-E_K)
         alpha_K1 = 1.02D0/(1.0D0+DEXP(0.2385D0*(Y(5)-E_K1-59.215D0)))
         beta_K1 = (0.49124D0*DEXP(0.08032D0*(Y(5)+5.476D0-E_K1))+1.0D0*    &
		 DEXP(0.06175D0*(Y(5)-(E_K1+594.31D0))))/(1.0D0+DEXP(-0.5143D0*     &
		 (Y(5)-E_K1+4.753D0)))
         K1_infinity = alpha_K1/(alpha_K1+beta_K1)
         i_K1 = g_K1*K1_infinity*(Y(5)-E_K1)
         Kp = 1.0D0/(1.0D0+DEXP((7.488D0-Y(5))/5.98D0))
         i_Kp = g_Kp*Kp*(Y(5)-E_Kp)
!        ionic current are expressed in microA/cm2:
		 y(5) = (i_Na+i_si+i_K+i_K1+i_Kp+i_b)

         RETURN
      END SUBROUTINE COMPUTE_I


!     --------------------------------------------------------------------------
