!=======================================================================
! CellML file:   luo_rudy_I_model_1991.cellml
! CellML model:  luo_rudy_I_model_1991
! Date and time: 19/01/2009 at 11:45:25
!-----------------------------------------------------------------------
! Conversion from CellML 1.0 to Fortran 77 (include) was done using COR (0.9.31.1165)
!    Copyright 2002-2009 Dr Alan Garny
!    http://COR.physiol.ox.ac.uk/ - COR@physiol.ox.ac.uk
!-----------------------------------------------------------------------
! http://www.CellML.org/
!=======================================================================

      !-----------------------------------------------------------------
      ! State variables
      !-----------------------------------------------------------------

      INTEGER(4) :: NB_OF_STATE_VARIABLES
      PARAMETER (NB_OF_STATE_VARIABLES = 8)

!      real(8) :: Y(8)
!      COMMON /Y/ Y
!      real(8) :: DY(8)
!      COMMON /DY/ DY
      ! 1: h (dimensionless) (in fast_sodium_current_h_gate)
      ! 2: j (dimensionless) (in fast_sodium_current_j_gate)
      ! 3: m (dimensionless) (in fast_sodium_current_m_gate)
      ! 4: Cai (millimolar) (in intracellular_calcium_concentration)
      ! 5: V (millivolt) (in membrane)
      ! 6: d (dimensionless) (in slow_inward_current_d_gate)
      ! 7: f (dimensionless) (in slow_inward_current_f_gate)
      ! 8: X (dimensionless) (in time_dependent_potassium_current_X_gate)

      CHARACTER YNames(8)*3
      COMMON /YNames/ YNames
      CHARACTER YUnits(8)*13
      COMMON /YUnits/ YUnits
      CHARACTER YComponents(8)*39
      COMMON /YComponents/ YComponents

      !-----------------------------------------------------------------
      ! Constants
      !-----------------------------------------------------------------

      real(8) :: E_b   ! millivolt (in background_current)
      real(8) :: g_b   ! milliS_per_cm2 (in background_current)
      real(8) :: g_Na   ! milliS_per_cm2 (in fast_sodium_current)
      real(8) :: Ki   ! millimolar (in ionic_concentrations)
      real(8) :: Ko   ! millimolar (in ionic_concentrations)
      real(8) :: Nai   ! millimolar (in ionic_concentrations)
      real(8) :: Nao   ! millimolar (in ionic_concentrations)
      real(8) :: C   ! microF_per_cm2 (in membrane)
      real(8) :: F   ! coulomb_per_mole (in membrane)
      real(8) :: R   ! joule_per_kilomole_kelvin (in membrane)
      real(8) :: T   ! kelvin (in membrane)
      real(8) :: stim_amplitude   ! microA_per_cm2 (in membrane)
      real(8) :: stim_duration   ! millisecond (in membrane)
      real(8) :: stim_end   ! millisecond (in membrane)
      real(8) :: stim_period   ! millisecond (in membrane)
      real(8) :: stim_start   ! millisecond (in membrane)
      real(8) :: g_Kp   ! milliS_per_cm2 (in plateau_potassium_current)
      real(8) :: PR_NaK   ! dimensionless (in time_dependent_potassium_current)

      COMMON /CONSTANTS001/ E_b, g_b, g_Na, Ki, Ko, Nai, Nao, C, F, R, T,    &
        stim_amplitude, stim_duration, stim_end, stim_period, stim_start,    &
        g_Kp, PR_NaK

      !-----------------------------------------------------------------
      ! Computed variables
      !-----------------------------------------------------------------

      real(8) :: i_b   ! microA_per_cm2 (in background_current)
      real(8) :: E_Na   ! millivolt (in fast_sodium_current)
      real(8) :: alpha_h   ! per_millisecond (in fast_sodium_current_h_gate)
      real(8) :: beta_h   ! per_millisecond (in fast_sodium_current_h_gate)
      real(8) :: alpha_j   ! per_millisecond (in fast_sodium_current_j_gate)
      real(8) :: beta_j   ! per_millisecond (in fast_sodium_current_j_gate)
      real(8) :: alpha_m   ! per_millisecond (in fast_sodium_current_m_gate)
      real(8) :: beta_m   ! per_millisecond (in fast_sodium_current_m_gate)
      real(8) :: i_Na   ! microA_per_cm2 (in fast_sodium_current)
      real(8) :: I_stim   ! microA_per_cm2 (in membrane)
      real(8) :: E_Kp   ! millivolt (in plateau_potassium_current)
      real(8) :: Kp   ! dimensionless (in plateau_potassium_current)
      real(8) :: i_Kp   ! microA_per_cm2 (in plateau_potassium_current)
      real(8) :: E_si   ! millivolt (in slow_inward_current)
      real(8) :: alpha_d   ! per_millisecond (in slow_inward_current_d_gate)
      real(8) :: beta_d   ! per_millisecond (in slow_inward_current_d_gate)
      real(8) :: alpha_f   ! per_millisecond (in slow_inward_current_f_gate)
      real(8) :: beta_f   ! per_millisecond (in slow_inward_current_f_gate)
      real(8) :: i_si   ! microA_per_cm2 (in slow_inward_current)
      real(8) :: E_K   ! millivolt (in time_dependent_potassium_current)
      real(8) :: alpha_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
      real(8) :: beta_X   ! per_millisecond (in time_dependent_potassium_current_X_gate)
      real(8) :: Xi   ! dimensionless (in time_dependent_potassium_current_Xi_gate)
      real(8) :: g_K   ! milliS_per_cm2 (in time_dependent_potassium_current)
      real(8) :: i_K   ! microA_per_cm2 (in time_dependent_potassium_current)
      real(8) :: E_K1   ! millivolt (in time_independent_potassium_current)
      real(8) :: K1_infinity   ! dimensionless (in time_independent_potassium_current_K1_gate)
      real(8) :: alpha_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
      real(8) :: beta_K1   ! per_millisecond (in time_independent_potassium_current_K1_gate)
      real(8) :: g_K1   ! milliS_per_cm2 (in time_independent_potassium_current)
      real(8) :: i_K1   ! microA_per_cm2 (in time_independent_potassium_current)

      COMMON /COMPVARS001/ i_b, E_Na, alpha_h, beta_h, alpha_j, beta_j,       &
        alpha_m, beta_m, i_Na, I_stim, E_Kp, Kp, i_Kp, E_si, alpha_d, beta_d, &
	alpha_f, beta_f, i_si, E_K, alpha_X, beta_X, Xi, g_K, i_K, E_K1,      &
	K1_infinity, alpha_K1, beta_K1, g_K1, i_K1

!=======================================================================
! End of file
!=======================================================================
